import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;
import java.awt.Point;
import java.awt.image.BufferedImage;
import greenfoot.core.WorldHandler;

/**
 * Palette, a Color choosing GUI, including:
 * <p>
 * 4 Sliders for changing the Red, Green, Blue, and Alpha components of a Color.
 * <p>
 * 24 memory slots. Clicking on the slots will change which one will be selected.
 * <p>
 * An "Enter" and "Cancel" Button that both remove the Palette from the World. The "Enter" Button provokes its listener to return that a Color ready, while the "Cancel" Button does not.
 * <p>
 * A "Find Color" Button that will "look" at the pixel where the mouse is (if inside the World). Clicking will then grab the Color at the mouse's position. Pressing escape will cancel it.
 * <p>
 * There is the possiblity that you have other Actors in the World that react to being clicked on, though you may wish to get a Color off of their images and don't want to envoke the Actors' behavior. To overcome this, inside of your World class constructor have a setPaintOrder() method with "Palette.Cover.class" as the first parameter.
 * <p>
 * Can drag the Palette around the world.
 * 
 * @author Taylor Born
 * @version November 2011
 */
public class Palette extends Actor
{
    private static GreenfootImage trans;
    private Slider redSlider = new Slider(150, 0, 255, 255, 1);
    private Slider greenSlider = new Slider(150, 0, 255, 255, 1);
    private Slider blueSlider = new Slider(150, 0, 255, 255, 1);
    private Slider alphaSlider = new Slider(150, 0, 255, 255, 1);
    private Color[][] colors = new Color[4][6];
    private Point selected = new Point(0, 0);
    private Point mouseDownAt = null;
    private Button btnEnter = new Button("Enter", new Point(44, 22));
    private Button btnCancel = new Button("Cancel", new Point(44, 22));
    private Button btnFind = new Button("Find Color", new Point(63, 22));
    private boolean ready = false;
    private Point lookingForColor = null;
    private Point lastMouse = new Point(-25, -25);
    
    private Cover cover = new Cover();

    static
    {
        trans = new GreenfootImage(64, 64);
        trans.setColor(Color.WHITE);
        trans.fill();
        trans.setColor(new Color(214, 214, 214));
        for (int i = 0; i < 4; i++)
            for (int k = 0; k < 4; k++)
                if (i % 2 == 0 ? k % 2 == 0 : k % 2 != 0)
                    trans.fillRect(k * 16, i * 16, 16, 16);
    }
    
    /**
     * A new Palette with some default colors stored.
     */
    public Palette()
    {
        for (int i = 1; i < colors.length; i++)
            for (int k = 0; k < colors[0].length; k++)
                colors[i][k] = Color.WHITE;
        colors[0][0] = Color.BLACK;
        colors[0][1] = Color.BLUE;
        colors[0][2] = new Color(153, 102, 51);
        colors[0][3] = Color.CYAN;
        colors[0][4] = Color.GREEN;
        colors[0][5] = Color.MAGENTA;
        colors[1][0] = Color.ORANGE;
        colors[1][1] = new Color(127, 0, 127);
        colors[1][2] = Color.RED;
        colors[1][3] = Color.YELLOW;
        redSlider.setValue(colors[0][0].getRed());
        greenSlider.setValue(colors[0][0].getGreen());
        blueSlider.setValue(colors[0][0].getBlue());
        alphaSlider.setValue(colors[0][0].getAlpha());
        act();
    }
    
    /**
     * A new Palette with some default colors stored, with Color given stored and active.
     * @param c The Color the Palette will have contained within its first memory slot (being the one first selected).
     */
    public Palette(Color c)
    {
        this();
        colors[0][0] = c;
        redSlider.setValue(colors[0][0].getRed());
        greenSlider.setValue(colors[0][0].getGreen());
        blueSlider.setValue(colors[0][0].getBlue());
        alphaSlider.setValue(colors[0][0].getAlpha());
        colors[1][4] = Color.BLACK;
    }
    
    public void act()
    {
        GreenfootImage pic = new GreenfootImage(300, 200);
        pic.setColor(Color.WHITE);
        pic.fill();
        pic.setColor(Color.BLACK);
        pic.drawRect(0, 0, pic.getWidth() - 1, pic.getHeight() - 1);
        pic.drawString("R", 5, 100 - 63 + 6);
        pic.drawString("G", 5, 100 - 21 + 6);
        pic.drawString("B", 5, 100 + 21 + 6);
        pic.drawString("A", 5, 100 + 63 + 6);
        
        if (lookingForColor != null)
        {
            if (lookingForColor.getX() > -1 && lookingForColor.getX() < getWorld().getWidth() && lookingForColor.getY() > -1 && lookingForColor.getY() < getWorld().getHeight())
            {
                int n = 175 + 125 / 2 - trans.getWidth() / 2 + 32 - (2 + 4 * 3);
                pic.drawRect(n - 1, 35 - 1, 4 * 7 + 1, 4 * 7 + 1);
                BufferedImage b = WorldHandler.getInstance().getSnapShot();
                for (int i = -3; i < 4; i++)
                    if (lookingForColor.getY() + i > -1 && lookingForColor.getY() + i < getWorld().getHeight())
                        for (int k = -3; k < 4; k++)
                            if (lookingForColor.getX() + k > -1 && lookingForColor.getX() + k < getWorld().getWidth())
                            {
                                pic.setColor(new Color(b.getRGB((int)lookingForColor.getX() + k, (int)lookingForColor.getY() + i)));
                                pic.fillRect(n + (k + 3) * 4, 35 + (i + 3) * 4, 4, 4);
                            }
                 pic.setColor(new Color(b.getRGB((int)lookingForColor.getX(), (int)lookingForColor.getY())));
                 pic.fillRect(n, 35 + 4 * 7 + 5, 4 * 7, 4 * 7);
                 pic.setColor(Color.BLACK);
                 pic.drawRect(n + 4 * 3 - 1, 35 + 4 * 3 - 1, 4 + 1, 4 + 1);
                 pic.drawRect(n - 1, 35 + 4 * 7 + 5 - 1, 4 * 7 + 1, 4 * 7 + 1);
            }
            if (Greenfoot.isKeyDown("escape"))
            {
                lookingForColor = null;
                getWorld().addObject(btnFind, getX() + 25 + 125 / 2, getY() - getImage().getHeight() / 2 + 11);
                getWorld().removeObject(cover);
            }
        }
        else
        {
            pic.drawImage(trans, 175 + 125 / 2 - trans.getWidth() / 2, 30);
            pic.drawRect(175 + 125 / 2 - trans.getWidth() / 2 - 1, 30 - 1, trans.getWidth() + 1, trans.getHeight() + 1);
            pic.setColor(colors[(int)selected.getY()][(int)selected.getX()]);
            pic.fillRect(175 + 125 / 2 - trans.getWidth() / 2, 30, trans.getWidth(), trans.getHeight());
        }
        MouseInfo mouse = Greenfoot.getMouseInfo();
        Point mousePos = null;
        if (Greenfoot.mousePressed(this))
            mousePos = new Point(mouse.getX() - (getX() - pic.getWidth() / 2), mouse.getY() - (getY() - pic.getHeight() / 2));
        if (Greenfoot.mouseClicked(null) && lookingForColor != null)
        {
            if (lookingForColor.getY() > -1 && lookingForColor.getY() < getWorld().getHeight() && lookingForColor.getX() > -1 && lookingForColor.getX() < getWorld().getWidth())
                setColor(new Color(WorldHandler.getInstance().getSnapShot().getRGB((int)lookingForColor.getX(), (int)lookingForColor.getY())));
            lookingForColor = null;
            getWorld().addObject(btnFind, getX() + 25 + 125 / 2, getY() - getImage().getHeight() / 2 + 11);
            getWorld().removeObject(cover);
        }
        if (btnFind.wasClicked())
        {
            lookingForColor = new Point((int)lastMouse.getX(), (int)lastMouse.getY());
            getWorld().removeObject(btnFind);
            getWorld().addObject(cover, getWorld().getWidth() / 2, getWorld().getHeight() / 2);
        }
        for (int i = 0; i < colors.length; i++)
            for (int k = 0; k < colors[0].length; k++)
            {
                
                Point p = new Point(175 + 125 / 2 - (int)(colors[0].length * 14 - 7 + 2) / 2 + k * 14 - 1, 200 - 40 - colors.length * 14 + i * 14 - 1);
                pic.setColor(colors[i][k]);
                pic.fillRect((int)p.getX(), (int)p.getY(), 10 + 1, 10 + 1);
                pic.setColor(Color.BLACK);
                pic.drawRect((int)p.getX(), (int)p.getY(), 10 + 1, 10 + 1);
                if (i == selected.getY() && k == selected.getX())
                {
                    pic.setColor(Color.RED);
                    pic.drawRect((int)p.getX() - 1, (int)p.getY() - 1, 10 + 3, 10 + 3);
                }
                if (mousePos != null && mousePos.getX() > p.getX() - 1 && mousePos.getX() < p.getX() + 11 + 2 && mousePos.getY() > p.getY() - 1 && mousePos.getY() < p.getY() + 11 + 2)
                {
                    selected = new Point(k, i);
                    redSlider.setValue(colors[i][k].getRed());
                    greenSlider.setValue(colors[i][k].getGreen());
                    blueSlider.setValue(colors[i][k].getBlue());
                    alphaSlider.setValue(colors[i][k].getAlpha());
                    mousePos = null;
                }
            }
        
        if (mousePos != null)
            mouseDownAt = new Point(mouse.getX(), mouse.getY());
        if (Greenfoot.mouseClicked(null) || Greenfoot.mouseDragEnded(null))
            mouseDownAt = null;
        if (Greenfoot.mouseDragged(null) || Greenfoot.mouseMoved(null))
        {
            if (mouseDownAt != null)
            {
                setLocation((int)(getX() + mouse.getX() - mouseDownAt.getX()), (int)(getY() + mouse.getY() - mouseDownAt.getY()));
                mouseDownAt = new Point(mouse.getX(), mouse.getY());
                redSlider.setLocation(getX() - 50, getY() - 63);
                greenSlider.setLocation(getX() - 50, getY() - 21);
                blueSlider.setLocation(getX() - 50, getY() + 21);
                alphaSlider.setLocation(getX() - 50, getY() + 63);
                btnEnter.setLocation(getX() + 25 + 125 / 2 - 25 + 1, getY() + 78);
                btnCancel.setLocation(getX() + 25 + 125 / 2 + 25 + 1, getY() + 78);
                if (btnFind.getWorld() != null)
                    btnFind.setLocation(getX() + 25 + 125 / 2, getY() - getImage().getHeight() / 2 + 11);
            }
            lastMouse = new Point(mouse.getX(), mouse.getY());
            if (lookingForColor != null)
                lookingForColor = new Point(mouse.getX(), mouse.getY());
        }
        setImage(pic);
        if (redSlider.hasChanged() || greenSlider.hasChanged() || blueSlider.hasChanged() || alphaSlider.hasChanged())
            colors[(int)selected.getY()][(int)selected.getX()] = new Color((int)redSlider.getValue(), (int)greenSlider.getValue(), (int)blueSlider.getValue(), (int)alphaSlider.getValue());
        boolean close = true;
        if (btnEnter.wasClicked())
            ready = true;
        else if (!btnCancel.wasClicked())
            close = false;
        if (close)
        {
            getWorld().removeObject(redSlider);
            getWorld().removeObject(greenSlider);
            getWorld().removeObject(blueSlider);
            getWorld().removeObject(alphaSlider);
            getWorld().removeObject(btnEnter);
            getWorld().removeObject(btnCancel);
            getWorld().removeObject(btnFind);
            getWorld().removeObject(this);
        }
        if (Greenfoot.isKeyDown("escape"))
        {}
    }
    public void addedToWorld(World world)
    {
        world.addObject(redSlider, getX() - 50, getY() - 63);
        world.addObject(greenSlider, getX() - 50, getY() - 21);
        world.addObject(blueSlider, getX() - 50, getY() + 21);
        world.addObject(alphaSlider, getX() - 50, getY() + 63);
        world.addObject(btnEnter, getX() + 25 + 125 / 2 - 25 + 1, getY() + 78);
        world.addObject(btnCancel, getX() + 25 + 125 / 2 + 25 + 1, getY() + 78);
        world.addObject(btnFind, getX() + 25 + 125 / 2, getY() - getImage().getHeight() / 2 + 11);
        cover.setImage(new GreenfootImage(world.getWidth(), world.getHeight()));
    }
    
    /**
     * Request the Color the Palette had last selected. To be called after method hasColorReady() returns true.
     * @return The Color the Palette had selected last.
     * @see #hasColorReady()
     */
    public Color getColor()
    {
        return colors[(int)selected.getY()][(int)selected.getX()];
    }
    
    /**
     * Set the current active memory slot to the given Color and updates Sliders.
     * @param c The Color to set as active Color.
     */
    public void setColor(Color c)
    {
        colors[(int)selected.getY()][(int)selected.getX()] = c;
        redSlider.setValue(c.getRed());
        greenSlider.setValue(c.getGreen());
        blueSlider.setValue(c.getBlue());
        alphaSlider.setValue(c.getAlpha());
    }
    
    /**
     * The action listener for the Palette. When returns true, the Palette's "Enter" Button has been pressed, hiding the Palette, and leaving the last Color selected made available from the getColor() method.
     * @return The Palette has a Color selected.
     * @see #getColor()
     */
    public boolean hasColorReady()
    {
        boolean r = ready;
        ready = false;
        return r;
    }
    class Cover extends Actor
    {
        
    }
}